from requests import request
from requests_toolbelt import MultipartEncoder
import json


class RequestHelper:

    @staticmethod
    def send_request(method, url, headers, params=None, data=None, multi_part=False, verify=True):
        r"""Sends a request.
        :param method: method of API.
        :param url: url path of API.
        :param headers: headers of request.
        :param params: params of request.
        :param data: (optional) Dictionary, list of tuples, bytes, or file-like
            object to send in the body of the :class:`Request`.
        :param multi_part:
        :param verify:
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        """
        request_headers = {'content-type': 'application/json'}
        request_headers.update(headers)
        if data and multi_part:
            data = MultipartEncoder(fields=data)
            request_headers['content-type'] = data.content_type
        elif data:
            data = json.dumps(data)
        response = request(method, url, headers=request_headers, params=params, data=data, verify=verify)
        return response
