from selenium import webdriver
from nextzencore.config.env_conf import EnvironmentConfiguration
from webdriver_manager.firefox import GeckoDriverManager
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.firefox.options import Options as FirefoxOptions


class Driver:

    @staticmethod
    def make_browser():

        if EnvironmentConfiguration.GRID:
            desired_capability = {
                'platform': EnvironmentConfiguration.PLATFORM,
                'browserName': EnvironmentConfiguration.BROWSER_NAME
            }
            grid_source = 'http://{host}:{port}/wd/hub'.format(host=EnvironmentConfiguration.GRID_HOST,
                                                               port=EnvironmentConfiguration.GRID_PORT)
            driver = webdriver.Remote(command_executor=grid_source, desired_capabilities=desired_capability)
        else:
            browser_name = EnvironmentConfiguration.BROWSER_NAME.lower()
            if browser_name == 'chrome':
                chrome_options = ChromeOptions()
                if EnvironmentConfiguration.HEADLESS:
                    chrome_options.add_argument("--headless")
                if EnvironmentConfiguration.NO_SANDBOX:
                    chrome_options.add_argument("--no-sandbox")
                driver = webdriver.Chrome(options=chrome_options)
            elif browser_name == 'firefox':
                firefox_options = FirefoxOptions()
                if EnvironmentConfiguration.HEADLESS:
                    firefox_options.headless = True
                driver = webdriver.Firefox(executable_path=GeckoDriverManager().install(), options=firefox_options)
            elif browser_name == 'edge':
                driver = webdriver.Edge()
            else:
                raise KeyError("The inputted browser is not supported")
        driver.set_page_load_timeout(EnvironmentConfiguration.PAGE_LOAD_TIMEOUT_SECONDS)
        driver.maximize_window()
        return driver
