from datetime import datetime, date, timedelta
from calendar import monthrange
from dateutil.relativedelta import *
from nextzencore.ui.config import constants as cc
import calendar
import json
import pytz
import dateutil
import dateutil.tz
import time


class DataHelper:

    @staticmethod
    def sort_alphabet_string_list(string_list, reverse=False):
        return sorted(string_list, key=lambda s: s.lower(), reverse=reverse)

    @staticmethod
    def convert_string_list_to_integer_list(string_list):
        return [int(item) for item in string_list]

    @staticmethod
    def convert_string_list_to_float_list(string_list):
        return [float(item) for item in string_list]

    @staticmethod
    def convert_string_list_with_multiple_delimiter_to_float_list(string_list):
        return [float(item.replace(',', '')) for item in string_list]

    @staticmethod
    def convert_string_with_multiple_delimiter_to_float(string):
        return float(string.replace(',', ''))

    @staticmethod
    def get_unique_string_by_time():
        return datetime.now().strftime('%Y%m%d-%H%M%S%f')

    @staticmethod
    def get_zone(offset):
        auth_file = open('./hooks/api/publisher_data/timezone_data.json')
        zones = json.load(auth_file)
        for zone in zones:
            if zone.get('offset') == offset:
                return pytz.timezone(zone.get('zone'))
        return None

    @staticmethod
    def get_local_timezone_name():
        local_timezone = dateutil.tz.tzlocal()
        return local_timezone.tzname(datetime.now(local_timezone))

    @staticmethod
    def convert_datetime_to_local(datetime_object, offset='+00:00'):
        """
        Convert datetime_object to utc datetime base on timezone offset
        :param datetime_object:
        :param offset:
        :return:
        """
        timezone = DataHelper.get_zone(offset)
        datetime_value = datetime_object.astimezone(timezone)
        return datetime_value.astimezone(pytz.utc)

    @staticmethod
    def convert_key_value_table_to_dictionary(table):
        if table is None:
            raise NameError(cc.RAISING_ERRORS['REQUIRED_DATA_TABLE'])
        result = {}
        for row in table:
            result[row[0]] = row[1]
        return result

    @staticmethod
    def str_to_bool(string):
        if string == 'True':
            return True
        elif string == 'False':
            return False
        else:
            raise ValueError

    @staticmethod
    def convert_status_string_to_boolean(status):
        refine_status = str(status).strip().lower()
        if refine_status == cc.PREDEFINED_FEATURE_VALUES['STATUS_ENABLED'].lower():
            return True
        if refine_status == cc.PREDEFINED_FEATURE_VALUES['STATUS_DISABLED'].lower():
            return False
        raise RuntimeError(cc.RAISING_ERRORS['UNDEFINED_STATUS'].format(status))

    @staticmethod
    def exclude_dictionary(dictionary, **criteria):

        excluded_keys = criteria.get('excluded_keys') if 'excluded_keys' in criteria else list()
        excluded_values = criteria.get('excluded_values') if 'excluded_values' in criteria else list()
        result = {}
        for key, value in dict(dictionary).items():
            if key in excluded_keys:
                continue
            if value in excluded_values:
                continue
            result[key] = value
        return result

    @staticmethod
    def switching_keys_and_values_in_a_dictionary(my_dict):
        """
        Switching keys and values in a dictionary in python
        :param my_dict:
        :return:
        Ex:
        my_dict = {2:3, 5:6, 8:9}
        return {3:2, 6:5, 9:8}
        """
        switching_dict = {v: k for k, v in my_dict.items()}
        return switching_dict

    @staticmethod
    def remove_underscore_and_capitalize_word(words):
        """
        Remove underscore and capitalize the first letter of word
        :param words:
        :return:
        """
        words_without_underscore = str(words).replace('_', ' ')
        return words_without_underscore.title()

    @staticmethod
    def get_list_from_string_list(str_array, separator=','):
        return [x.strip() for x in str_array.split(separator)]

    @staticmethod
    def filter_list_dict_by_dict(list_dict, filter_dict):
        """
        Filter list of dict by filter_dict.
        :param list_dict:
        :param filter_dict:
        :return: Return list of dict match filter_dict
        Ex: list_dict=[{'id': 1, 'v': 11},
                       {'id': 1, 'v': 111},
                       {'id': 2, 'v': 22},
                       {'id': 3, 'v': 33}]
        Case1:
             filter_dict={'id': 1, 'v': 11}
          => return [{'id': 1, 'v': 11}]

        Case2:
             filter_dict={'id': 1}
          => return [{'id': 1, 'v': 11}, {'id': 1, 'v': 111}]
        """

        result_list = []
        for item in list_dict:
            count_match = len(filter_dict.keys())
            count_check = 0
            if set(filter_dict.keys()).issubset(set(item.keys())):
                for k, v in filter_dict.items():
                    if v == item[k]:
                        count_check += 1
            if count_check == count_match:
                result_list.append(item)
            else:
                continue
        return result_list

    @staticmethod
    def get_value_of_key_from_list_dict(list_dict, selected_key):
        """
        Get value of key from list dict
        :param list_dict:
        :param selected_key:
        :return: list of values that have selected_key in each dict
        Ex: list_dict=[{'id': 1, 'v': 11},
                       {'id': 2, 'v': 22},
                       {'id': 3, 'v': 33}]
             selected_key='id'
          => return [1,2,3]
        """
        result_list = []
        for dict_item in list_dict:
            if selected_key in dict(dict_item).keys():
                result_list.append(dict_item.get(selected_key))
        return result_list

    @staticmethod
    def get_value_from_nested_dict(data_dict, map_str, separator='.'):
        """
            Get value from nested dict
        :param data_dict:
        :param map_str: key1.key2.key3
        :param separator: for split map string of path. Default: '.'
        :return:
        Ex: list_dict={{'id1': {'value':'1'}},
                       {'id2': {'value':'2'}},
                       {'id3': {'value':'3'}}
             map_str='id1.value'
          => return '1'
        """
        map_list = str(map_str).split(separator)
        pointer = data_dict
        value = None
        try:
            for ind in range(len(map_list)):
                key = map_list[ind]
                if isinstance(pointer, list):
                    if ind == len(map_list) - 1:
                        value = pointer[int(key[1:-1])]
                        return value
                    pointer = pointer[int(key[1:-1])]
                elif ind == len(map_list) - 1:
                    value = pointer[key]
                    return value
                elif key in pointer:
                    pointer = pointer[key]
            return value
        except (KeyError, TypeError, IndexError):
            return None

    @staticmethod
    def set_value_to_nested_dict(dict_data, map_str, value, separator='.'):
        """
            Set value to nested dict
        :param dict_data:
        :param map_str: key1.key2.key3
        :param value:
        :param separator: for split map string of path. Default: '.'
        :return:
        """
        map_list = str(map_str).split(separator)
        pointer = dict_data
        for ind in range(len(map_list)):
            key = map_list[ind]
            if '[' in key:
                if ind == len(map_list) - 1:
                    pointer[int(key[1:-1])] = value
                    return
                pointer = pointer[int(key[1:-1])]
            elif ind == len(map_list) - 1:
                pointer[key] = value
                return
            elif key in pointer:
                pointer = pointer[key]
            else:
                pointer[key] = {}
                pointer = pointer[key]

    @staticmethod
    def convert_list_to_string(list_tmp, separator=','):
        """
        Convert list to string
        :param list_tmp:
        :param separator:
        :return:
        Ex: list_str=['1','2','3','4','5']
          => return '1,2,3,4,5'
        """
        result_str = list_tmp
        if isinstance(list_tmp, list):
            result_str = str(separator.join(list_tmp))
        return result_str

    @staticmethod
    def format_unicode_to_right_format_string(invalid_format_unicode):
        """
        Convert unicode string to right format string
        :param invalid_format_unicode:
        :return:
        """
        return str(invalid_format_unicode.encode('ascii', 'ignore').decode('ascii')).replace('\n', '')

    @staticmethod
    def format_unicode_to_right_format_string_of_list(invalid_format_unicode_list):
        """
        Convert unicode string to right format string of all elements in list
        :param invalid_format_unicode_list:
        :return:
        """
        result_list = []
        for item in invalid_format_unicode_list:
            result_list.append(DataHelper.format_unicode_to_right_format_string(item))
        return result_list

    @staticmethod
    def replace_old_to_new_value_from_string_list(string_list, old, new=''):
        return [item.replace(old, new) for item in string_list]

    @staticmethod
    def wait_until_match_condition(time_out, error, method, *args):
        """
        Wait until method return True. This is often applied to wait the the returned result of API
        :param time_out:
        :param error: error when timeout
        :param method:
        :param args: all arguments for method
        :return:
        """
        count_loop = 0
        interval_wait = 5
        while True:
            condition = method(*args)
            if condition:
                return True
            time.sleep(interval_wait)
            count_loop += 1
            if (interval_wait * count_loop) > time_out:
                raise RuntimeError(error)

    @staticmethod
    def remove_space_in_element_of_string_list(string_list):
        return [element.replace(' ', '') for element in string_list]

    @staticmethod
    def first_lower(s):
        return s[0].lower() + s[1:]

    @staticmethod
    def remove_duplicate_dict_from_a_list(dict_list):
        new_dict_list = []

        for item in dict_list:
            if item not in new_dict_list:
                new_dict_list.append(item)
        return new_dict_list

    @staticmethod
    def remove_key_from_a_dict_list(dict_list, key):
        for item in dict_list:
            item.pop(key)
        return dict_list

    @staticmethod
    def remove_leading_and_ending_spaces_from_a_list(string_list):
        return [element.strip() for element in string_list]

    @staticmethod
    def convert_standard_date_time_format(date_obj):
        return date_obj.strftime('%m/%d/%Y')

    @staticmethod
    def get_start_date(date_range):
        return (datetime.now() - timedelta(days=int(cc.PREDEFINED_DATE_RANGE[date_range]['start_date_delta']))).date()

    @staticmethod
    def get_end_date(date_range):
        return (datetime.now() - timedelta(days=int(cc.PREDEFINED_DATE_RANGE[date_range]['end_date_delta']))).date()

    @staticmethod
    def get_first_date_of_previous_week(given_date):
        start_date = given_date + timedelta(-given_date.weekday(), weeks=-1)
        return start_date.date()

    @staticmethod
    def get_first_date_of_last_month(given_date):
        last_date = datetime(given_date.year, given_date.month, 1) - timedelta(1)
        first_date = last_date.replace(day=1)
        return first_date.date()

    @staticmethod
    def get_first_date_of_last_year(given_date):
        first_date = datetime(given_date.year, 1, 1) - relativedelta(years=1)
        return first_date.date()

    @staticmethod
    def get_this_week(given_date, include_today):
        """
        Default first of weekday is 1 (Monday) however we support Sunday is as first of weekday so we need plus 1
        :param given_date:today
        :return:
        """
        first_date = given_date - timedelta(days=given_date.weekday()) - timedelta(1)
        if include_today:
            last_date = given_date
        else:
            last_date = given_date - timedelta(1)
        return first_date, last_date

    @staticmethod
    def get_last_week(given_date):
        """
        Default first of weekday is 1 (Monday) however we support Sunday is as first of weekday so we need plus 1
        :param date:
        :return:
        """
        first_date = given_date + timedelta(-given_date.weekday(), weeks=-1) - timedelta(1)
        last_date = first_date + timedelta(days=6)
        if last_date.month > given_date.month:
            last_date = given_date
        return first_date, last_date

    @staticmethod
    def get_this_month(given_date, include_today):
        """
        Get first and last date of this month
        :param given_date:
        :return:
        """
        first_date = given_date.replace(day=1)
        if include_today:
            last_date = given_date
        else:
            last_date = given_date - timedelta(1)
        return first_date, last_date

    @staticmethod
    def get_last_month(given_date):
        """
        Get first and last date of this month
        :param given_date:
        :return:
        """
        first_day_this_month = date.today().replace(day=1)
        last_month = (first_day_this_month - timedelta(days=1)).month
        this_year = given_date.year
        year = this_year - 1 if last_month == 12 else this_year
        first_date = date(year, last_month, 1)
        last_date = date(year, last_month, monthrange(year, last_month)[1])
        return first_date, last_date

    @staticmethod
    def get_quarter(given_date):
        return int((given_date.month - 1) / 3 + 1)

    @staticmethod
    def get_this_quarter(given_date, include_today):
        """
        last_month_of_current_quarter = 3 * current_quarter
        last_day = datetime(date.year, last_month_of_current_quarter,
                            calendar.monthrange(year, last_month_of_current_quarter)[1])
        :param given_date:
        :return:
        """
        current_quarter = DataHelper.get_quarter(given_date)
        first_month_of_current_quarter = 3 * current_quarter - 2
        first_day = datetime(given_date.year, first_month_of_current_quarter, 1)
        if include_today:
            last_date = given_date
        else:
            last_date = given_date - timedelta(days=1)
        return first_day, last_date

    @staticmethod
    def get_last_quarter(given_date):
        this_quarter = DataHelper.get_quarter(given_date)
        this_year = given_date.year
        if this_quarter == 1:
            last_quarter = 4
            year = this_year - 1
        else:
            last_quarter = this_quarter - 1
            year = this_year
        first_date = date(year, 3 * last_quarter - 2, 1)
        last_date = date(year, 3 * last_quarter, monthrange(year, 3 * last_quarter)[1])
        return first_date, last_date

    @staticmethod
    def get_this_year(given_date, include_today):
        """
        Get the first and last date of This Year
        :param given_date:
        :return:
        """

        first_day = datetime.now().date().replace(month=1, day=1)
        if include_today:
            last_date = given_date
        else:
            last_date = given_date - timedelta(days=1)
        return first_day, last_date

    @staticmethod
    def get_last_year(given_date):
        """
        Get the first and last date of Last Year
        :param given_date:
        :return:
        """
        last_year = given_date.year - 1
        first_date = given_date.replace(year=last_year, month=1, day=1)
        last_day = calendar.monthrange(year=last_year, month=12)[1]
        last_date = first_date.replace(day=last_day, month=12)
        return first_date, last_date
